import * as tslib_1 from "tslib";
// import compose from 'recompose/compose';
// import mapProps from 'recompose/mapProps';
// import setDisplayName from 'recompose/setDisplayName';
// import withState from 'recompose/withState';
import { compose, mapProps, setDisplayName, withState, } from 'recompose';
/**
 * Given an error from yup validation, turn it into form errors
 */
export function yupToFormErrors(yupError) {
    var errors = {};
    for (var _i = 0, _a = yupError.inner; _i < _a.length; _i++) {
        var err = _a[_i];
        if (!errors[err.path]) {
            errors[err.path] = err.message;
        }
    }
    return errors;
}
/**
 * Given a FormState, return a `touched` value with all of the fields touched.
 */
export function touchAllFields(fields) {
    var touched = {};
    for (var _i = 0, _a = Object.keys(fields); _i < _a.length; _i++) {
        var k = _a[_i];
        touched[k] = true;
    }
    return touched;
}
export function validateFormData(data, schema, setErrors) {
    var validateData = {};
    for (var k in data) {
        if (data.hasOwnProperty(k)) {
            var key = String(k);
            validateData[key] = data[key] !== ''
                ? data[key]
                : undefined;
        }
    }
    return schema.validate(validateData, { abortEarly: false }).then(function () {
        setErrors({});
        return true;
    }, function (err) {
        setErrors(yupToFormErrors(err));
        return false;
    });
}
export default function Formik(_a) {
    var displayName = _a.displayName, _b = _a.mapPropsToFormState, mapPropsToFormState = _b === void 0 ? function (formProps) { return formProps; } : _b, _c = _a.mapFormStateToPayload, mapFormStateToPayload = _c === void 0 ? function (formPartialState) {
        // in this case State and Payload are the same.
        var payload = formPartialState;
        return payload;
    } : _c, validationSchema = _a.validationSchema, handleSubmit = _a.handleSubmit;
    return compose(setDisplayName(displayName), withState('form', 'setForm', function (props) { return mapPropsToFormState(props); }), withState('errors', 'setErrors', {}), withState('touched', 'setTouched', {}), withState('isSubmitting', 'setSubmitting', false), mapProps(function (_a) {
        var // onSubmit,
        form = _a.form, errors = _a.errors, touched = _a.touched, isSubmitting = _a.isSubmitting, setErrors = _a.setErrors, setForm = _a.setForm, setTouched = _a.setTouched, setSubmitting = _a.setSubmitting, rest = tslib_1.__rest(_a, ["form", "errors", "touched", "isSubmitting", "setErrors", "setForm", "setTouched", "setSubmitting"]);
        return (tslib_1.__assign({ onChange: function (e) {
                e.persist();
                var _a = e.target, type = _a.type, name = _a.name, value = _a.value, checked = _a.checked;
                var val = /number|range/.test(type)
                    ? parseFloat(value)
                    : /checkbox/.test(type)
                        ? checked
                        : /radio/.test(type) // is this needed?
                            ? value
                            : value;
                // Set changed fields as touched
                setTouched(tslib_1.__assign({}, touched, (_b = {}, _b[name] = true, _b)));
                // Set form fields by name
                setForm(tslib_1.__assign({}, form, (_c = {}, _c[name] = val, _c)));
                // Validate against schema
                validateFormData(tslib_1.__assign({}, form, (_d = {}, _d[name] = val, _d)), validationSchema, setErrors);
                var _b, _c, _d;
            }, onSubmit: function (e) {
                e.preventDefault();
                setTouched(touchAllFields(form));
                validateFormData(form, validationSchema, setErrors).then(function (isValid) {
                    console.log('isValid:' + isValid);
                    if (isValid) {
                        handleSubmit(mapFormStateToPayload(form));
                    }
                });
            }, setForm: setForm,
            setErrors: setErrors,
            setSubmitting: setSubmitting,
            errors: errors,
            isSubmitting: isSubmitting,
            touched: touched }, rest, form));
    }));
}
